@@ -146,6 +146,7 @@ extern "C" {
     #define LBFGS_FLOAT 64
     #define real_t double
     #define exp_t exp
+    #define log_t log
     #define cblas_tdot cblas_ddot
     #define cblas_tcopy cblas_dcopy
     #define cblas_taxpy cblas_daxpy
@@ -166,6 +167,7 @@ extern "C" {
     #define LBFGS_FLOAT 32
     #define real_t float
     #define exp_t expf
+    #define log_t logf
     #define cblas_tdot cblas_sdot
     #define cblas_tcopy cblas_scopy
     #define cblas_taxpy cblas_saxpy
@@ -493,20 +495,7 @@ void factors_implicit_chol
     real_t *restrict Xa, int_t ixB[], size_t nnz,
     real_t lam,
     real_t *restrict precomputedBtB, int_t ld_BtB,
-    bool zero_out,
-    real_t *restrict buffer_real_t,
-    bool force_add_diag
-);
-void factors_implicit
-(
-    real_t *restrict a_vec, int_t k,
-    real_t *restrict B, size_t ldb,
-    real_t *restrict Xa, int_t ixB[], size_t nnz,
-    real_t lam,
-    real_t *restrict precomputedBtB, int_t ld_BtB,
-    bool zero_out, bool use_cg, int_t max_cg_steps,
-    real_t *restrict buffer_real_t,
-    bool force_add_diag
+    real_t *restrict buffer_real_t
 );
 real_t fun_grad_Adense
 (
@@ -673,7 +662,7 @@ int_t fit_most_popular
     int_t ixA[], int_t ixB[], real_t *restrict X, size_t nnz,
     real_t *restrict Xfull,
     real_t *restrict weight,
-    bool implicit, bool adjust_weight,
+    bool implicit, bool adjust_weight, bool apply_log_transf,
     real_t *restrict w_main_multiplier,
     int_t nthreads
 );
@@ -1266,7 +1255,7 @@ int_t fit_collective_implicit_als
     int_t k_main, int_t k_user, int_t k_item,
     real_t w_main, real_t w_user, real_t w_item,
     real_t *restrict w_main_multiplier,
-    real_t alpha, bool adjust_weight,
+    real_t alpha, bool adjust_weight, bool apply_log_transf,
     int_t niter, int_t nthreads, bool verbose, bool handle_interrupt,
     bool use_cg, int_t max_cg_steps, bool finalize_chol,
     bool precompute_for_predictions,
@@ -1341,6 +1330,7 @@ int_t factors_collective_implicit_single
     int_t k, int_t k_user, int_t k_item, int_t k_main,
     real_t lam, real_t alpha, real_t w_main, real_t w_user,
     real_t w_main_multiplier,
+    bool apply_log_transf,
     real_t *restrict BeTBe,
     real_t *restrict BtB,
     real_t *restrict BeTBeChol
@@ -1390,6 +1380,7 @@ int_t factors_collective_implicit_multiple
     int_t k, int_t k_user, int_t k_item, int_t k_main,
     real_t lam, real_t alpha, real_t w_main, real_t w_user,
     real_t w_main_multiplier,
+    bool apply_log_transf,
     real_t *restrict BeTBe,
     real_t *restrict BtB,
     real_t *restrict BeTBeChol,
@@ -1493,6 +1484,7 @@ int_t topN_new_collective_implicit
     int_t k, int_t k_user, int_t k_item, int_t k_main,
     real_t lam, real_t alpha, real_t w_main, real_t w_user,
     real_t w_main_multiplier,
+    bool apply_log_transf,
     real_t *restrict BeTBe,
     real_t *restrict BtB,
     real_t *restrict BeTBeChol,
@@ -1574,6 +1566,7 @@ int_t predict_X_new_collective_implicit
     int_t k, int_t k_user, int_t k_item, int_t k_main,
     real_t lam, real_t alpha, real_t w_main, real_t w_user,
     real_t w_main_multiplier,
+    bool apply_log_transf,
     real_t *restrict BeTBe,
     real_t *restrict BtB,
     real_t *restrict BeTBeChol
@@ -1822,7 +1815,8 @@ int_t fit_offsets_als
     real_t lam,
     real_t *restrict U, int_t p,
     real_t *restrict II, int_t q,
-    bool implicit, bool NA_as_zero_X, real_t alpha,
+    bool implicit, bool NA_as_zero_X,
+    real_t alpha, bool apply_log_transf,
     int_t niter,
     int_t nthreads, bool use_cg,
     int_t max_cg_steps, bool finalize_chol,
@@ -1832,7 +1826,6 @@ int_t fit_offsets_als
     real_t *restrict Bm_plus_bias,
     real_t *restrict precomputedBtB,
     real_t *restrict precomputedTransBtBinvBt
-
 );
 int_t fit_offsets_explicit_als
 (
@@ -1873,7 +1866,7 @@ int_t fit_offsets_implicit_als
     real_t lam,
     real_t *restrict U, int_t p,
     real_t *restrict II, int_t q,
-    real_t alpha,
+    real_t alpha, bool apply_log_transf,
     int_t niter,
     int_t nthreads, bool use_cg,
     int_t max_cg_steps, bool finalize_chol,
@@ -1921,6 +1914,7 @@ int_t factors_offsets_implicit_single
     real_t *restrict C_bias,
     int_t k, int_t n,
     real_t lam, real_t alpha,
+    bool apply_log_transf,
     real_t *restrict precomputedBtB,
     real_t *restrict output_a
 );
@@ -1959,6 +1953,7 @@ int_t factors_offsets_implicit_multiple
     real_t *restrict C_bias,
     int_t k, int_t n,
     real_t lam, real_t alpha,
+    bool apply_log_transf,
     real_t *restrict precomputedBtB,
     int_t nthreads
 );
@@ -2021,6 +2016,7 @@ int_t topN_new_offsets_implicit
     real_t *restrict C_bias,
     int_t k,
     real_t lam, real_t alpha,
+    bool apply_log_transf,
     real_t *restrict precomputedBtB,
     /* inputs for topN */
     int_t *restrict include_ix, int_t n_include,
@@ -2088,6 +2084,7 @@ int_t predict_X_new_offsets_implicit
     real_t *restrict C_bias,
     int_t k,
     real_t lam, real_t alpha,
+    bool apply_log_transf,
     real_t *restrict precomputedBtB
 );
 int_t fit_content_based_lbfgs
